@@ -25,6 +25,9 @@ import time
 
 from keystoneauth1.access import service_catalog as keystone_sc
 from keystoneauth1 import exceptions as keystone_exc
+from keystoneauth1 import identity as ksa_identity
+from keystoneauth1 import session as ksa_session
+from keystoneauth1 import token_endpoint as ksa_token_endpoint
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import units
@@ -33,7 +36,7 @@ from glance_store import capabilities
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LI
+from glance_store.i18n import _, _LE, _LI, _LW
 import glance_store.location
 
 try:
@@ -53,7 +56,7 @@ LOG = logging.getLogger(__name__)
 
 _CINDER_OPTS = [
     cfg.StrOpt('cinder_catalog_info',
-               default='volumev2::publicURL',
+               default='volumev3::publicURL',
                help="""
 Information to match when looking for cinder in the service catalog.
 
@@ -79,6 +82,8 @@ Related options:
     * cinder_store_user_name
     * cinder_store_project_name
     * cinder_store_password
+    * cinder_store_project_domain_name
+    * cinder_store_user_domain_name
 
 """),
     cfg.StrOpt('cinder_endpoint_template',
@@ -104,6 +109,8 @@ Related options:
     * cinder_store_user_name
     * cinder_store_project_name
     * cinder_store_password
+    * cinder_store_project_domain_name
+    * cinder_store_user_domain_name
     * cinder_catalog_info
 
 """),
@@ -215,6 +222,8 @@ Related options:
     * cinder_store_user_name
     * cinder_store_password
     * cinder_store_project_name
+    * cinder_store_project_domain_name
+    * cinder_store_user_domain_name
 
 """),
     cfg.StrOpt('cinder_store_user_name',
@@ -222,8 +231,9 @@ Related options:
                help="""
 User name to authenticate against cinder.
 
-This must be used with all the following related options. If any of these are
-not specified, the user of the current context is used.
+This must be used with all the following non-domain-related options.
+If any of these are not specified (except domain-related options),
+the user of the current context is used.
 
 Possible values:
     * A valid user name
@@ -232,14 +242,33 @@ Related options:
     * cinder_store_auth_address
     * cinder_store_password
     * cinder_store_project_name
+    * cinder_store_project_domain_name
+    * cinder_store_user_domain_name
+
+"""),
+    cfg.StrOpt('cinder_store_user_domain_name',
+               default='Default',
+               help="""
+Domain of the user to authenticate against cinder.
+
+Possible values:
+    * A valid domain name for the user specified by ``cinder_store_user_name``
+
+Related options:
+    * cinder_store_auth_address
+    * cinder_store_password
+    * cinder_store_project_name
+    * cinder_store_project_domain_name
+    * cinder_store_user_name
 
 """),
     cfg.StrOpt('cinder_store_password', secret=True,
                help="""
 Password for the user authenticating against cinder.
 
-This must be used with all the following related options. If any of these are
-not specified, the user of the current context is used.
+This must be used with all the following related options.
+If any of these are not specified (except domain-related options),
+the user of the current context is used.
 
 Possible values:
     * A valid password for the user specified by ``cinder_store_user_name``
@@ -248,6 +277,8 @@ Related options:
     * cinder_store_auth_address
     * cinder_store_user_name
     * cinder_store_project_name
+    * cinder_store_project_domain_name
+    * cinder_store_user_domain_name
 
 """),
     cfg.StrOpt('cinder_store_project_name',
@@ -258,8 +289,9 @@ Project name where the image volume is stored in cinder.
 If this configuration option is not set, the project in current context is
 used.
 
-This must be used with all the following related options. If any of these are
-not specified, the project of the current context is used.
+This must be used with all the following related options.
+If any of these are not specified (except domain-related options),
+the user of the current context is used.
 
 Possible values:
     * A valid project name
@@ -268,6 +300,25 @@ Related options:
     * ``cinder_store_auth_address``
     * ``cinder_store_user_name``
     * ``cinder_store_password``
+    * ``cinder_store_project_domain_name``
+    * ``cinder_store_user_domain_name``
+
+"""),
+    cfg.StrOpt('cinder_store_project_domain_name',
+               default='Default',
+               help="""
+Domain of the project where the image volume is stored in cinder.
+
+Possible values:
+    * A valid domain name of the project specified by
+      ``cinder_store_project_name``
+
+Related options:
+    * ``cinder_store_auth_address``
+    * ``cinder_store_user_name``
+    * ``cinder_store_password``
+    * ``cinder_store_project_domain_name``
+    * ``cinder_store_user_domain_name``
 
 """),
     cfg.StrOpt('rootwrap_config',
@@ -350,6 +401,34 @@ Possible values:
 """),
 ]
 
+CINDER_SESSION = None
+
+
+def _reset_cinder_session():
+    global CINDER_SESSION
+    CINDER_SESSION = None
+
+
+def get_cinder_session(conf):
+    global CINDER_SESSION
+    if not CINDER_SESSION:
+        auth = ksa_identity.V3Password(
+            password=conf.cinder_store_password,
+            username=conf.cinder_store_user_name,
+            user_domain_name=conf.cinder_store_user_domain_name,
+            project_name=conf.cinder_store_project_name,
+            project_domain_name=conf.cinder_store_project_domain_name,
+            auth_url=conf.cinder_store_auth_address
+        )
+        if conf.cinder_api_insecure:
+            verify = False
+        elif conf.cinder_ca_certificates_file:
+            verify = conf.cinder_ca_certificates_file
+        else:
+            verify = True
+        CINDER_SESSION = ksa_session.Session(auth=auth, verify=verify)
+    return CINDER_SESSION
+
 
 class StoreLocation(glance_store.location.StoreLocation):
 
@@ -405,38 +484,26 @@ class Store(glance_store.driver.Store):
 
     def configure_add(self):
         """
-        Configure the Store to use the stored configuration options
-        Any store that needs special configuration should implement
-        this method. If the store was not able to successfully configure
-        itself, it should raise `exceptions.BadStoreConfiguration`
-        :raises: `exceptions.BadStoreConfiguration` if multiple stores are
-                 defined and particular store wasn't able to configure
-                 successfully
-        :raises: `exceptions.BackendException` if single store is defined and
-                 it wasn't able to configure successfully
+        Check to verify if the volume types configured for the cinder store
+        exist in deployment and if not, log a warning.
         """
-        if self.backend_group:
-            cinder_volume_type = self.store_conf.cinder_volume_type
-            if cinder_volume_type:
-                # NOTE: `cinder_volume_type` is configured, check
-                # configured volume_type is available in cinder or not
-                cinder_client = self.get_cinderclient()
-                try:
-                    # We don't even need the volume type object, as long
-                    # as this returns clean, we know the name is good.
-                    cinder_client.volume_types.find(name=cinder_volume_type)
-                    # No need to worry NoUniqueMatch as volume type name is
-                    # unique
-                except cinder_exception.NotFound:
-                    reason = _("Invalid `cinder_volume_type %s`"
-                               % cinder_volume_type)
-                    if len(self.conf.enabled_backends) > 1:
-                        LOG.error(reason)
-                        raise exceptions.BadStoreConfiguration(
-                            store_name=self.backend_group, reason=reason)
-                    else:
-                        LOG.critical(reason)
-                        raise exceptions.BackendException(reason)
+        cinder_volume_type = self.store_conf.cinder_volume_type
+        if cinder_volume_type:
+            # NOTE: `cinder_volume_type` is configured, check
+            # configured volume_type is available in cinder or not
+            cinder_client = self.get_cinderclient()
+            try:
+                # We don't even need the volume type object, as long
+                # as this returns clean, we know the name is good.
+                cinder_client.volume_types.find(name=cinder_volume_type)
+                # No need to worry about a NoUniqueMatch as volume type name
+                # is unique
+            except cinder_exception.NotFound:
+                reason = (_LW("Invalid `cinder_volume_type %s`"
+                              % cinder_volume_type))
+                LOG.warning(reason)
+            except cinder_exception.ClientException:
+                pass
 
     def is_image_associated_with_store(self, context, volume_id):
         """
@@ -488,15 +555,18 @@ class Store(glance_store.driver.Store):
         else:
             user_overriden = self.is_user_overriden()
 
+        session = get_cinder_session(self.store_conf)
+
         if user_overriden:
             username = self.store_conf.cinder_store_user_name
-            password = self.store_conf.cinder_store_password
-            project = self.store_conf.cinder_store_project_name
             url = self.store_conf.cinder_store_auth_address
+            # use auth that is already in the session
+            auth = None
         else:
             username = context.user_id
-            password = context.auth_token
             project = context.project_id
+            # noauth extracts user_id:project_id from auth_token
+            token = context.auth_token or '%s:%s' % (username, project)
 
             if self.store_conf.cinder_endpoint_template:
                 template = self.store_conf.cinder_endpoint_template
@@ -516,23 +586,17 @@ class Store(glance_store.driver.Store):
                     reason = _("Failed to find Cinder from a service catalog.")
                     raise exceptions.BadStoreConfiguration(store_name="cinder",
                                                            reason=reason)
+            auth = ksa_token_endpoint.Token(endpoint=url, token=token)
 
         c = cinderclient.Client(
-            username, password, project, auth_url=url,
+            session=session, auth=auth,
             region_name=self.store_conf.cinder_os_region_name,
-            insecure=self.store_conf.cinder_api_insecure,
-            retries=self.store_conf.cinder_http_retries,
-            cacert=self.store_conf.cinder_ca_certificates_file)
+            retries=self.store_conf.cinder_http_retries)
 
         LOG.debug(
             'Cinderclient connection created for user %(user)s using URL: '
             '%(url)s.', {'user': username, 'url': url})
 
-        # noauth extracts user_id:project_id from auth_token
-        if not user_overriden:
-            c.client.auth_token = context.auth_token or '%s:%s' % (username,
-                                                                   project)
-        c.client.management_url = url
         return c
 
     @contextlib.contextmanager
@@ -640,7 +704,7 @@ class Store(glance_store.driver.Store):
             connection_info = volume.initialize_connection(volume, properties)
             conn = connector.InitiatorConnector.factory(
                 connection_info['driver_volume_type'], root_helper,
-                conn=connection_info)
+                conn=connection_info, use_multipath=use_multipath)
             if connection_info['driver_volume_type'] == 'nfs':
                 if volume.encrypted:
                     volume.unreserve(volume)
@@ -829,8 +893,8 @@ class Store(glance_store.driver.Store):
 
         self._check_context(context, require_tenant=True)
         client = self.get_cinderclient(context)
-        os_hash_value = hashlib.new(str(hashing_algo))
-        checksum = hashlib.md5()
+        os_hash_value = utils.get_hasher(hashing_algo, False)
+        checksum = utils.get_hasher('md5', False)
         bytes_written = 0
         size_gb = int(math.ceil(float(image_size) / units.Gi))
         if size_gb == 0:
@@ -849,8 +913,18 @@ class Store(glance_store.driver.Store):
             LOG.info(_LI("Since image size is zero, we will be doing "
                          "resize-before-write for each GB which "
                          "will be considerably slower than normal."))
-        volume = client.volumes.create(size_gb, name=name, metadata=metadata,
-                                       volume_type=volume_type)
+        try:
+            volume = client.volumes.create(size_gb, name=name,
+                                           metadata=metadata,
+                                           volume_type=volume_type)
+        except cinder_exception.NotFound:
+            LOG.error(_LE("Invalid volume type %s configured. Please check "
+                          "the `cinder_volume_type` configuration parameter."
+                          % volume_type))
+            msg = (_("Failed to create image-volume due to invalid "
+                     "`cinder_volume_type` configured."))
+            raise exceptions.BackendException(msg)
+
         volume = self._wait_volume_status(volume, 'creating', 'available')
         size_gb = volume.size
 
