@@ -26,9 +26,9 @@ import tempfile
 import time
 import uuid
 
-from cinderclient.v3 import client as cinderclient
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 
 from glance_store import exceptions
@@ -60,37 +60,36 @@ class TestCinderStore(base.StoreBaseTest,
         fake_sc = [{u'endpoints': [{u'publicURL': u'http://foo/public_url'}],
                     u'endpoints_links': [],
                     u'name': u'cinder',
-                    u'type': u'volumev2'}]
+                    u'type': u'volumev3'}]
         self.context = FakeObject(service_catalog=fake_sc,
                                   user_id='fake_user',
                                   auth_token='fake_token',
                                   project_id='fake_project')
         self.hash_algo = 'sha256'
+        cinder._reset_cinder_session()
 
     def test_get_cinderclient(self):
         cc = self.store.get_cinderclient(self.context)
-        self.assertEqual('fake_token', cc.client.auth_token)
-        self.assertEqual('http://foo/public_url', cc.client.management_url)
+        self.assertEqual('fake_token', cc.client.auth.token)
+        self.assertEqual('http://foo/public_url', cc.client.auth.endpoint)
 
-    def test_get_cinderclient_with_user_overriden(self):
+    def _test_get_cinderclient_with_user_overriden(self):
         self.config(cinder_store_user_name='test_user')
         self.config(cinder_store_password='test_password')
         self.config(cinder_store_project_name='test_project')
         self.config(cinder_store_auth_address='test_address')
         cc = self.store.get_cinderclient(self.context)
-        self.assertIsNone(cc.client.auth_token)
-        self.assertEqual('test_address', cc.client.management_url)
+        self.assertEqual('test_project', cc.client.session.auth.project_name)
+        self.assertEqual('Default', cc.client.session.auth.project_domain_name)
+        return cc
+
+    def test_get_cinderclient_with_user_overriden(self):
+        self._test_get_cinderclient_with_user_overriden()
 
     def test_get_cinderclient_with_user_overriden_and_region(self):
         self.config(cinder_os_region_name='test_region')
-        fake_client = FakeObject(client=FakeObject(auth_token=None))
-        with mock.patch.object(cinderclient, 'Client',
-                               return_value=fake_client) as mock_client:
-            self.test_get_cinderclient_with_user_overriden()
-            mock_client.assert_called_once_with(
-                'test_user', 'test_password', 'test_project',
-                auth_url='test_address', cacert=None, insecure=False,
-                region_name='test_region', retries=3)
+        cc = self._test_get_cinderclient_with_user_overriden()
+        self.assertEqual('test_region', cc.client.region_name)
 
     def test_temporary_chown(self):
         class fake_stat(object):
@@ -188,7 +187,7 @@ class TestCinderStore(base.StoreBaseTest,
                 mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
-                                  side_effect=fake_factory):
+                                  side_effect=fake_factory) as fake_conn_obj:
 
             with mock.patch.object(connector,
                                    'get_connector_properties') as mock_conn:
@@ -223,6 +222,9 @@ class TestCinderStore(base.StoreBaseTest,
                         None, 'glance_store', attach_mode,
                         host_name=socket.gethostname())
                     fake_volumes.detach.assert_called_once_with(fake_volume)
+                    fake_conn_obj.assert_called_once_with(
+                        mock.ANY, root_helper, conn=mock.ANY,
+                        use_multipath=multipath_supported)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -333,7 +335,8 @@ class TestCinderStore(base.StoreBaseTest,
         expected_size = size_kb * units.Ki
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
-        expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_checksum = md5(expected_file_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = 'cinder://%s' % fake_volume.id
         fake_client = FakeObject(auth_token=None, management_url=None)
@@ -411,3 +414,23 @@ class TestCinderStore(base.StoreBaseTest,
 
     def test_set_url_prefix(self):
         self.assertEqual('cinder://', self.store._url_prefix)
+
+    def test_configure_add(self):
+
+        def fake_volume_type(name):
+            if name != 'some_type':
+                raise cinder.cinder_exception.NotFound(code=404)
+
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type))
+            self.config(cinder_volume_type='some_type')
+            # If volume type exists, no exception is raised
+            self.store.configure_add()
+            # setting cinder_volume_type to non-existent value will log a
+            # warning
+            self.config(cinder_volume_type='some_random_type')
+            with mock.patch.object(cinder, 'LOG') as mock_log:
+                self.store.configure_add()
+                mock_log.warning.assert_called_with(
+                    "Invalid `cinder_volume_type some_random_type`")
