@@ -15,8 +15,6 @@
 
 import contextlib
 import errno
-import io
-import math
 import os
 from unittest import mock
 
@@ -100,14 +98,15 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             user_id='admin_user',
             auth_token='admin_token',
             project_id='admin_project')
+        cinder._reset_cinder_session()
 
     def test_location_url_prefix_is_set(self):
         self.assertEqual("cinder://cinder1", self.store.url_prefix)
 
     def test_get_cinderclient(self):
         cc = self.store.get_cinderclient(self.context)
-        self.assertEqual('fake_token', cc.client.auth_token)
-        self.assertEqual('http://foo/public_url', cc.client.management_url)
+        self.assertEqual('fake_token', cc.client.auth.token)
+        self.assertEqual('http://foo/public_url', cc.client.auth.endpoint)
 
     def test_get_cinderclient_with_user_overriden(self):
         self.config(cinder_store_user_name='test_user', group="cinder1")
@@ -115,16 +114,14 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         self.config(cinder_store_project_name='test_project', group="cinder1")
         self.config(cinder_store_auth_address='test_address', group="cinder1")
         cc = self.store.get_cinderclient(self.context)
-        self.assertIsNone(cc.client.auth_token)
-        self.assertEqual('test_address', cc.client.management_url)
+        self.assertEqual('Default', cc.client.session.auth.project_domain_name)
+        self.assertEqual('test_project', cc.client.session.auth.project_name)
 
     def test_get_cinderclient_legacy_update(self):
         cc = self.store.get_cinderclient(self.fake_admin_context,
                                          legacy_update=True)
-        self.assertEqual('admin_token', cc.client.auth_token)
-        self.assertEqual('admin_user', cc.client.user)
-        self.assertEqual('admin_project', cc.client.projectid)
-        self.assertEqual('http://foo/public_url', cc.client.management_url)
+        self.assertEqual('admin_token', cc.client.auth.token)
+        self.assertEqual('http://foo/public_url', cc.client.auth.endpoint)
 
     def test_temporary_chown(self):
         class fake_stat(object):
@@ -221,18 +218,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                 mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
-                                  side_effect=fake_factory
-                                  ) as fake_conn_obj, \
-                mock.patch.object(socket,
-                                  'gethostname') as mock_get_host, \
-                mock.patch.object(socket,
-                                  'getaddrinfo') as mock_get_host_ip:
-
-            fake_host = 'fake_host'
-            fake_addr_info = [[0, 1, 2, 3, ['127.0.0.1']]]
-            fake_ip = fake_addr_info[0][4][0]
-            mock_get_host.return_value = fake_host
-            mock_get_host_ip.return_value = fake_addr_info
+                                  side_effect=fake_factory) as fake_conn_obj:
 
             with mock.patch.object(connector,
                                    'get_connector_properties') as mock_conn:
@@ -242,8 +228,8 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                     do_open()
 
                 mock_conn.assert_called_once_with(
-                    root_helper, fake_ip, multipath_supported,
-                    enforce_multipath, host=fake_host)
+                    root_helper, socket.gethostname(), multipath_supported,
+                    enforce_multipath)
                 fake_connector.connect_volume.assert_called_once_with(mock.ANY)
                 fake_connector.disconnect_volume.assert_called_once_with(
                     mock.ANY, fake_devinfo)
@@ -442,7 +428,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual(expected_image_size, image_size)
 
     def _test_cinder_add(self, fake_volume, volume_file, size_kb=5,
-                         verifier=None, backend="cinder1", fail_resize=False):
+                         verifier=None, backend="cinder1"):
         expected_image_id = str(uuid.uuid4())
         expected_size = size_kb * units.Ki
         expected_file_contents = b"*" * expected_size
@@ -462,11 +448,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
-                                  side_effect=fake_open), \
-                mock.patch.object(
-                    cinder.Store, '_wait_resize_device') as mock_wait_resize:
-            if fail_resize:
-                mock_wait_resize.side_effect = exceptions.BackendException()
+                                  side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
             loc, size, checksum, metadata = self.store.add(expected_image_id,
@@ -541,43 +523,3 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                      size=1)
         volume_file = six.BytesIO()
         self._test_cinder_add(fake_volume, volume_file, backend="cinder2")
-
-    def test__get_device_size(self):
-        fake_data = b"fake binary data"
-        fake_len = int(math.ceil(float(len(fake_data)) / units.Gi))
-        fake_file = io.BytesIO(fake_data)
-        dev_size = cinder.Store._get_device_size(fake_file)
-        self.assertEqual(fake_len, dev_size)
-
-    @mock.patch.object(time, 'sleep')
-    def test__wait_resize_device_resized(self, mock_sleep):
-        fake_vol = mock.MagicMock()
-        fake_vol.size = 2
-        fake_file = io.BytesIO(b"fake binary data")
-        with mock.patch.object(
-                cinder.Store, '_get_device_size') as mock_get_dev_size:
-            mock_get_dev_size.side_effect = [1, 2]
-            cinder.Store._wait_resize_device(fake_vol, fake_file)
-
-    @mock.patch.object(time, 'sleep')
-    def test__wait_resize_device_fails(self, mock_sleep):
-        fake_vol = mock.MagicMock()
-        fake_vol.size = 2
-        fake_file = io.BytesIO(b"fake binary data")
-        with mock.patch.object(
-                cinder.Store, '_get_device_size',
-                return_value=1):
-            self.assertRaises(
-                exceptions.BackendException,
-                cinder.Store._wait_resize_device,
-                fake_vol, fake_file)
-
-    def test_cinder_add_fail_resize(self):
-        volume_file = io.BytesIO()
-        fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
-                                     status='available',
-                                     size=1)
-        self.assertRaises(exceptions.BackendException,
-                          self._test_cinder_add, fake_volume, volume_file,
-                          fail_resize=True)
-        fake_volume.delete.assert_called_once()
